==> distfun-0.3-1-src.zip: distfun/src/genrand/setgmn.c <==
/* setgmn.f -- translated by f2c (version 20100827).
You must link the resulting object file with libf2c:
on Microsoft Windows system, link with libf2c.lib;
on Linux or Unix systems, link with .../path/to/libf2c.a -lm
or, if you install libf2c.a in a standard place, with -lf2c -lm
-- in that order, at the end of the command line, as in
cc *.o -lf2c -lm
Source for libf2c is in /netlib/f2c/libf2c.zip, e.g.,

http://www.netlib.org/f2c/libf2c.zip
*/

#include "genrand.h"

/* Subroutine */ 
int setgmn_(double *meanv, double *covm, int *
	ldcovm, int *p, double *parm, int *ierr)
{
	/* System generated locals */
	int covm_dim1, covm_offset, i__1, i__2;

	/* Local variables */
	static int i__, j, io;
	static float wte;
	static int info;
	static int icount;

	/*      SUBROUTINE setgmn(meanv,covm,p,parm) */
	/*     JJV changed this routine to take leading dimension of COVM */
	/*     JJV argument and pass it to SPOFA, making it easier to use */
	/*     JJV if the COVM which is used is contained in a larger matrix */
	/*     JJV and to make the routine more consistent with LINPACK. */
	/*     JJV Changes are in comments, declarations, and the call to SPOFA. */
	/* ********************************************************************** */

	/*     SUBROUTINE SETGMN( MEANV, COVM, LDCOVM, P, PARM) */
	/*            SET Generate Multivariate Normal random deviate */


	/*                              Function */


	/*      Places P, MEANV, and the Cholesky factoriztion of COVM */
	/*      in PARM for GENMN. */


	/*                              Arguments */


	/*     MEANV --> Mean vector of multivariate normal distribution. */
	/*                                        DOUBLE PRECISION MEANV(P) */

	/*     COVM   <--> (Input) Covariance   matrix    of  the  multivariate */
	/*                 normal distribution.  This routine uses only the */
	/*                 (1:P,1:P) slice of COVM, but needs to know LDCOVM. */

	/*                 (Output) Destroyed on output */
	/*                                        DOUBLE PRECISION COVM(LDCOVM,P) */

	/*     LDCOVM --> Leading actual dimension of COVM. */
	/*                                        INTEGER LDCOVM */

	/*     P     --> Dimension of the normal, or length of MEANV. */
	/*                                        INTEGER P */

	/*     PARM <-- Array of parameters needed to generate multivariate */
	/*                normal deviates (P, MEANV and Cholesky decomposition */
	/*                of COVM). */
	/*                1 : 1                - P */
	/*                2 : P + 1            - MEANV */
	/*                P+2 : P*(P+3)/2 + 1  - Cholesky decomposition of COVM */
	/*                                             DOUBLE PRECISION PARM(P*(P+3)/2 + 1) */

	/* ********************************************************************** */
	/*     .. Scalar Arguments .. */
	/*      INTEGER p */
	/*      include 'stack.h' */
	/*     .. */
	/*     .. Array Arguments .. */
	/*      DOUBLE PRECISION covm(p,p),meanv(p),parm(p* (p+3)/2+1) */
	/*     .. */
	/*     .. Local Scalars .. */
	/*     .. */

	/*     .. */
	/*     .. Executable Statements .. */


	/*     TEST THE INPUT */

	/* Parameter adjustments */
	--parm;
	covm_dim1 = *ldcovm;
	covm_offset = 1 + covm_dim1;
	covm -= covm_offset;
	--meanv;

	/* Function Body */
	/* L10: */
	parm[1] = (double) (*p);

	/*     PUT P AND MEANV INTO PARM */

	i__1 = *p + 1;
	for (i__ = 2; i__ <= i__1; ++i__) {
		parm[i__] = meanv[i__ - 1];
		/* L20: */
	}

	/*      Cholesky decomposition to find A s.t. trans(A)*(A) = COVM */

	/*      CALL spofa(covm,p,p,info) */
	spofa_(&covm[covm_offset], ldcovm, p, &info);
	*ierr = 0;
	if (! (info != 0)) {
		goto L30;
	}
	genrand_messageprint("SET Generate Multivariate Normal Random Number: COV not positive definite");
	*ierr = 1;
	return 0;
L30:
	icount = *p + 1;

	/*     PUT UPPER HALF OF A, WHICH IS NOW THE CHOLESKY FACTOR, INTO PARM */
	/*          COVM(1,1) = PARM(P+2) */
	/*          COVM(1,2) = PARM(P+3) */
	/*                    : */
	/*          COVM(1,P) = PARM(2P+1) */
	/*          COVM(2,2) = PARM(2P+2)  ... */

	i__1 = *p;
	for (i__ = 1; i__ <= i__1; ++i__) {
		i__2 = *p;
		for (j = i__; j <= i__2; ++j) {
			++icount;
			parm[icount] = covm[i__ + j * covm_dim1];
			/* L40: */
		}
		/* L50: */
	}
	return 0;

} /* setgmn_ */

==> distfun-0.5-1-src.zip: distfun/src/genrand/setgmn.c <==
/* genrand_setgmn.f -- translated by f2c (version 20100827).
with manual tuning by Michael Baudin (2012)
*/

#include "genrand.h"
#include "genrand_private.h"

int genrand_setgmn(double *meanv, double *covm, int *ldcovm, int *p, double *parm, int *ierr)
{
	int covm_dim1, covm_offset, i__1, i__2;

	static int i__, j, io;
	static float wte;
	static int info;
	static int icount;

	/*     JJV changed this routine to take leading dimension of COVM */
	/*     JJV argument and pass it to genrand_spofa, making it easier to use */
	/*     JJV if the COVM which is used is contained in a larger matrix */
	/*     JJV and to make the routine more consistent with LINPACK. */
	/*     JJV Changes are in comments, declarations, and the call to genrand_spofa. */

	/*     .. Array Arguments .. */
	/*      DOUBLE PRECISION covm(p,p),meanv(p),parm(p* (p+3)/2+1) */

	/*     TEST THE INPUT */

	/* Parameter adjustments */
	--parm;
	covm_dim1 = *ldcovm;
	covm_offset = 1 + covm_dim1;
	covm -= covm_offset;
	--meanv;

	/* Function Body */
	/* L10: */
	parm[1] = (double) (*p);

	/*     PUT P AND MEANV INTO PARM */

	i__1 = *p + 1;
	for (i__ = 2; i__ <= i__1; ++i__) {
		parm[i__] = meanv[i__ - 1];
		/* L20: */
	}

	/*      Cholesky decomposition to find A s.t. trans(A)*(A) = COVM */

	/*      CALL genrand_spofa(covm,p,p,info) */
	genrand_spofa(&covm[covm_offset], ldcovm, p, &info);
	*ierr = 0;
	if (! (info != 0)) {
		goto L30;
	}
	genrand_messageprint("SET Generate Multivariate Normal Random Number: COV not positive definite");
	*ierr = 1;
	return 0;
L30:
	icount = *p + 1;

	/*     PUT UPPER HALF OF A, WHICH IS NOW THE CHOLESKY FACTOR, INTO PARM */
	/*          COVM(1,1) = PARM(P+2) */
	/*          COVM(1,2) = PARM(P+3) */
	/*                    : */
	/*          COVM(1,P) = PARM(2P+1) */
	/*          COVM(2,2) = PARM(2P+2)  ... */

	i__1 = *p;
	for (i__ = 1; i__ <= i__1; ++i__) {
		i__2 = *p;
		for (j = i__; j <= i__2; ++j) {
			++icount;
			parm[icount] = covm[i__ + j * covm_dim1];
			/* L40: */
		}
		/* L50: */
	}
	return 0;

} /* genrand_setgmn_ */
